/**
 * @(#) GljMSJDirect.java
 */


package gl4java.system;

/**
 * The base toolkit class for MS-JVM JDirect calls !
 *
 * The methods in this class are used by gl4java.GLContext !
 * Because this class MUST compiled with the MS-JVM Java Compiler
 * I have decided to put it out of gl4java.GLContext,
 * although this can be a security hole ...
 * But we must be able to use GL4Java without MS-JVM bytecode.
 *
 * The Tag @security(checkDllCalls=off) makes us able to
 * use these methods outside of these class,
 * without a security check all the time.
 * The security check is performed at link-time !
 * 
 * This is the same for gl4java.GLUFuncMSJDirect
 * and gl4java.GLFuncMSJDirect !
 *
 * @see gl4java.GLContext
 * @version 	1.00
 * @author      Ron Cemer, Sven Goethel
 * 
 */

/** @security(checkDllCalls=off) */
public class GljMSJDirect extends Object 
{

    /* JDirect DLL functions */

    public static final void OGLWindowMsgPump()
    { OGLWindowMsgPumpJDirect(); }


    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native void OGLWindowMsgPumpJDirect();

    public static final int createOGLWindowNative(int hwndParent, 
    					          int x, int y, 
						  int width, 
						  int height)
    { return createOGLWindowNativeJDirect(hwndParent, x,y, width, height); }

    /**
     * @dll.import("GL4JavaGljMSJDirect", auto)
     */
    private static native int createOGLWindowNativeJDirect(int hwndParent, 
    					                  int x, int y, 
							  int width, 
							  int height);

    public static final void destroyOGLWindowNative(int hdc)
    { destroyOGLWindowNativeJDirect(hdc); }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native void destroyOGLWindowNativeJDirect(int hdc);

    public static final void moveOGLWindowNative(int hdc, int x, int y, int width, int height)
    { moveOGLWindowNativeJDirect(hdc, x, y, width, height); }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native void moveOGLWindowNativeJDirect(int hdc, int x, int y, int width, int height);

    public static final int openOpenGLNative(boolean verbose, int hdc,
    					     boolean doubleBuffer,
					     boolean stereoView,
					     int stencilBits,
					     int sharedGLContextNative)
    {
    	return openOpenGLNativeJDirect( verbose, hdc, 
				        doubleBuffer, stereoView,
					stencilBits,
				        sharedGLContextNative);
    }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native int openOpenGLNativeJDirect(boolean verbose, int hdc,
    						     boolean doubleBuffer,
						     boolean stereoView,
						     int stencilBits,
					             int sharedGLContextNative);

    public static final boolean testDoubleBuffer ( int hdc)
    { return testDoubleBufferJDirect(hdc); }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native boolean testDoubleBufferJDirect ( int hdc);

    public static final boolean testStereoView ( int hdc)
    { return testStereoViewJDirect(hdc); }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native boolean testStereoViewJDirect ( int hdc);

    public static final int testStencilBits( int hdc)
    { return testStencilBitsJDirect(hdc); }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native int testStencilBitsJDirect ( int hdc);

    public static final boolean gljMakeCurrentNative(int hdc, int gc, 
    				                    boolean freeContextFirst)
    { return gljMakeCurrentNativeJDirect(hdc, gc, freeContextFirst); }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native boolean gljMakeCurrentNativeJDirect(int hdc, int gc, 
    				                    boolean freeContextFirst);

    public static final boolean gljFreeNative()
    { return gljFreeNativeJDirect(); }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native boolean gljFreeNativeJDirect();

    public static final boolean gljDestroyNative(int gc)
    { return gljDestroyNativeJDirect(gc); }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native boolean gljDestroyNativeJDirect(int gc);

    public static final boolean gljSwapNative(int hdc, boolean doubleBuffer)
    { return gljSwapNativeJDirect(hdc, doubleBuffer); }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native boolean gljSwapNativeJDirect(int hdc, boolean doubleBuffer);

    public static final int gljResizeNative(int hdc, int width, int height)
    { return gljResizeNativeJDirect(hdc, width, height); }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native int gljResizeNativeJDirect(int hdc, int width, int height);

    public static final boolean gljCheckGLTypesNative()
    { return gljCheckGLTypesNativeJDirect(); }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native boolean gljCheckGLTypesNativeJDirect();

    public static final int gljGetNativeLibVendorNative(int sizeofbuffer, StringBuffer buffer)
    { return gljGetNativeLibVendorNativeJDirect(sizeofbuffer, buffer); }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native int gljGetNativeLibVendorNativeJDirect(int sizeofbuffer, StringBuffer buffer);

    public static final int gljGetNativeLibVersionNative(int sizeofbuffer, StringBuffer buffer)
    { return gljGetNativeLibVersionNativeJDirect(sizeofbuffer, buffer); }

    /**
     * @dll.import("GL4JavaGljMSJDirect")
     */
    private static native int gljGetNativeLibVersionNativeJDirect(int sizeofbuffer, StringBuffer buffer);



    /* End of JDirect DLL functions */

}

