package gl4java.utils.textures;

import gl4java.*;

import java.awt.*;
import java.awt.image.*;
import java.awt.Color.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;
import java.net.*;

import com.sixlegs.image.png.*;

/**
 * This is Class implements the PNG texture-loader
 * while using the png-library in package
 * "com.sixlegs.image.png" !
 *
 * @see IOTextureLoader
 * @see TextureLoader
 */   
public class PngTextureLoader
extends IOTextureLoader
{
	public PngTextureLoader(GLFunc gl, GLUFunc glu)
	{
		super(gl, glu);
	}

	protected boolean readTexture(InputStream is)
	{
	  try {
            int len;
            PngImage png = new PngImage(is);
            
            imageWidth = png.getWidth();
	    imageHeight = png.getHeight();

            // Read entire PNG image (doesn't throw exceptions)
            int[] iPixels = new int[imageWidth * imageHeight];

            PixelGrabber pp=new PixelGrabber(png,
                                             0,0,
                                             imageWidth, imageHeight,
                                             iPixels,
                                             0,
                                             imageWidth);
            try 
            {
                pp.grabPixels();
            } 
            catch (InterruptedException e) 
            {
                System.err.println("interrupted waiting for pixel!");
		error=true;
                return false;
            }
            if ((pp.getStatus() & ImageObserver.ABORT) != 0) 
            {
                System.err.println("image fetch aborted or errored");
		error=true;
                return false;
            }

            // bitDepth beachten
            switch(png.getColorType())
            {
                case PngImage.COLOR_TYPE_RGB:
                    glFormat=GL_RGB;
                    break;
                case PngImage.COLOR_TYPE_RGB_ALPHA:
                    glFormat=GL_RGBA;
                    break;
                default:
		    error=true;
                    System.err.println("unsupported format");
                    return false;
            };
           
            pixel=new byte[imageWidth * imageHeight * getComponents()];
            
            byte alpha=0;
            byte red=0;
            byte green=0;
            byte blue=0;
            int offset=0;
	    int aPixel;
            for(int y=imageHeight-1; y>=0; y--)
	    {
              for(int x=0;x<imageWidth;x++)
              {
	        aPixel = iPixels[y*imageWidth + x];

                if(glFormat==GL_RGBA)
                   alpha =new Integer( (aPixel  >> 24) & 0xff ).byteValue();
                red   =new Integer( (aPixel  >> 16) & 0xff ).byteValue();
                green =new Integer( (aPixel  >>  8) & 0xff ).byteValue();
                blue  =new Integer( (aPixel       ) & 0xff ).byteValue();
                
                pixel[offset++]=red;
                pixel[offset++]=green;
                pixel[offset++]=blue;
                if(glFormat==GL_RGBA)
                    pixel[offset++]=alpha;
              }
            }

	    setTextureSize();
	    return true;

	   } catch (Exception e) {
	        System.out.println("An exception occured, while loading a PngTexture");
	        System.out.println(e);
	        error=true;
	   }
	   return false;
	}

}

