package gl4java.utils.textures;

import gl4java.*;

import java.io.*;
import java.net.*;

/**
 * This abstract Class defines the interface
 * for ALL texture Grabbers !
 * The TextureGrabber's implementations are used to
 * save the pixels of the GL Context to a file !
 *
 * @see TextureTool
 * @see	GLImageCanvas
 */   
public abstract class TextureGrabber
implements GLEnum, GLUEnum
{
  protected byte[] pixels=null;
  protected int xpos;
  protected int ypos;
  protected int width;
  protected int height;
  protected GLFunc gl;

  public TextureGrabber(GLFunc gl)
  {
    this.gl=gl;
  }

  /**
   * Grab the pixels outta the OpenGL Frambuffer
   *
   * @param source the frambuffer source (like glReadBuffer), 
   *		   can be: GL_FRONT, GL_BACK, ....
   * @param x      the xpos
   * @param y      the ypos
   * @param w      the width
   * @param h      the height
   *
   * @see GLFunc#glReadBuffer
   */
  public void grabPixels(int source, int x, int y, int w, int h) 
  {
    xpos=x;
    ypos=y;
    width=w;
    height=h;
    pixels=new byte[w * h * 3];
    
    //setup pixel storage modes for data coming into client (our) memory
    gl.glPixelStorei(GL_PACK_ALIGNMENT, 1); /* byte alignment */
    gl.glPixelStorei(GL_PACK_ROW_LENGTH, w);
          
    //get viewport data
    gl.glReadBuffer(source);
    gl.glReadPixels(x, y, w, h, GL_RGB, GL_UNSIGNED_BYTE, pixels);    

    //reset pixel storage modes
    gl.glPixelStorei(GL_PACK_ALIGNMENT, 4);
    gl.glPixelStorei(GL_PACK_ROW_LENGTH, 0);
  }

  public boolean write2File(String fname)
  {
    try {
    	    OutputStream os= new java.io.FileOutputStream(fname);
	    return write2File(os);
    } catch (Exception ex) {
    	System.out.println("TGATextureGrabber.write2File <"+
		fname+"> failed !\n"+ex);
    }
    return false;
  }

  public boolean write2File(URL base, String uri)
  {
    try {
	    URL url = new URL (base, uri);
	    URLConnection urlcon = url.openConnection();
	    urlcon.setDoOutput(true);
	    OutputStream os = urlcon.getOutputStream();
	    return write2File(os);
    } catch (Exception ex) {
    	System.out.println("TGATextureGrabber.write2File <"+
		base+" / "+uri+"> failed !\n"+ex);
    }
    return false;
  }

  public abstract boolean write2File(OutputStream os);
}
  
